//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: XmlModify.cpp,v 1.6 2004/09/02 16:37:42 jsnelson Exp $
//

#include "dbxml_config.h"
#include "dbxml/XmlPortability.hpp"
#include "dbxml/XmlModify.hpp"
#include "dbxml/XmlQueryExpression.hpp"
#include "dbxml/XmlTransaction.hpp"
#include "QueryExpression.hpp"
#include "Modify.hpp"
#include "dbxml/XmlResults.hpp"

using namespace DbXml;

XmlModify::XmlModify(Modify *m)
  : modify_(m)
{
  modify_->acquire();
}

XmlModify::XmlModify(const XmlModify &o)
  : modify_(o.modify_)
{
  modify_->acquire();
}

XmlModify &XmlModify::operator=(const XmlModify &o)
{
  if(modify_ != o.modify_) {
    modify_->release();
    modify_ = o.modify_;
    modify_->acquire();
  }

  return *this;
}

XmlModify::~XmlModify()
{
	modify_->release();
}

void XmlModify::setNewEncoding(const std::string &newEncoding)
{
	modify_->setNewEncoding(&newEncoding);
}

void XmlModify::addInsertBeforeStep(const XmlQueryExpression &selectionExpr, XmlObject type, const std::string &name, const std::string &content)
{
  modify_->addStep(new InsertBeforeStep(selectionExpr, type, name, content, modify_->getMemMgr(), modify_->getDOMParser()));
}

void XmlModify::addInsertAfterStep(const XmlQueryExpression &selectionExpr, XmlObject type, const std::string &name, const std::string &content)
{
  modify_->addStep(new InsertAfterStep(selectionExpr, type, name, content, modify_->getMemMgr(), modify_->getDOMParser()));
}

void XmlModify::addAppendStep(const XmlQueryExpression &selectionExpr, XmlObject type, const std::string &name, const std::string &content, int location)
{
  modify_->addStep(new AppendStep(selectionExpr, type, name, content, location, modify_->getMemMgr(), modify_->getDOMParser()));
}

void XmlModify::addUpdateStep(const XmlQueryExpression &selectionExpr, const std::string &content)
{
  modify_->addStep(new UpdateStep(selectionExpr, content));
}

void XmlModify::addRemoveStep(const XmlQueryExpression &selectionExpr)
{
  modify_->addStep(new RemoveStep(selectionExpr));
}

void XmlModify::addRenameStep(const XmlQueryExpression &selectionExpr, const std::string &newName)
{
  modify_->addStep(new RenameStep(selectionExpr, newName));
}

unsigned int XmlModify::execute(XmlValue &toModify, XmlQueryContext &context, XmlUpdateContext &uc) const
{
  return modify_->execute(0, toModify, context, uc);
}

unsigned int XmlModify::execute(XmlResults &toModify, XmlQueryContext &context, XmlUpdateContext &uc) const
{
  return modify_->execute(0, toModify, context, uc);
}

unsigned int XmlModify::execute(XmlTransaction &txn, XmlValue &toModify, XmlQueryContext &context, XmlUpdateContext &uc) const
{
  return modify_->execute(txn, toModify, context, uc);
}

unsigned int XmlModify::execute(XmlTransaction &txn, XmlResults &toModify, XmlQueryContext &context, XmlUpdateContext &uc) const
{
  return modify_->execute(txn, toModify, context, uc);
}

