//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: AddDocumentCommand.cpp,v 1.3 2004/09/06 14:01:13 jsnelson Exp $
//

#include "AddDocumentCommand.hpp"

using namespace DbXml;
using namespace std;

string AddDocumentCommand::getCommandName() const
{
  return "adddocument";
}

string AddDocumentCommand::getBriefHelp() const
{
  return "Add a document to the default container";
}

string AddDocumentCommand::getMoreHelp() const
{
  return
    string("Takes arguments of the name prefix of the documents to add,") +
    string("and an xquery command that return the content of the documents.\n") +
    string("A document is added for each result returned by the expression.\n") +
    string("This command uses the XmlContainer::putDocument() method, with\n") +
    string("the DBXML_GEN_NAME flag specified.\n");
}

void AddDocumentCommand::execute(Args &args, Environment &env)
{
  if(args.size() != 3) {
    throw CommandException("Wrong number of arguments");
  }
  env.testContainer();
  env.deleteResults();

  if(env.txn()) {
	  XmlTransaction myTxn = env.childTransaction();
	  env.results() = new XmlResults(env.db().query(myTxn, args[2], env.context()));

	  XmlValue value;
	  while(env.results()->next(value)) {
		  string v = value.asString();
		  string name = env.container()->putDocument(myTxn, args[1], v, env.uc(), DBXML_GEN_NAME);
		  if(env.verbose()) cout << "Document added, name = " << name << ", content = " << v << endl;
	  }

	  myTxn.commit();
  }
  else {
	  env.results() = new XmlResults(env.db().query(args[2], env.context()));

	  XmlValue value;
	  while(env.results()->next(value)) {
		  string v = value.asString();
		  string name = env.container()->putDocument(args[1], v, env.uc(), DBXML_GEN_NAME);
		  if(env.verbose()) cout << "Document added, name = " << name << ", content = " << v << endl;
	  }

  }
}

