//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: RemoveCommand.cpp,v 1.4 2004/10/18 22:52:13 gmf Exp $
//

#include "RemoveCommand.hpp"

using namespace DbXml;
using namespace std;

string RemoveCommand::getCommandName() const
{
  return "remove";
}

string RemoveCommand::getBriefHelp() const
{
  return "Remove nodes from the documents in the last results returned";
}

string RemoveCommand::getMoreHelp() const
{
  return
    string("Takes the path to the nodes to remove as an argument. The results\n") +
    string("of the modifications are left in the default results. This command\n") +
    string("creates an XmlModify object using XmlManager::createModify(),\n") +
    string("then adds a remove step to it using XmlModify::addRemoveStep(). It\n") +
    string("then executes using XmlModify::execute().");
}

void RemoveCommand::execute(Args &args, Environment &env)
{
  if(args.size() != 2) {
    throw CommandException("Wrong number of arguments");
  }
  env.testResults();

  if(env.verbose()) cout << "Removing nodes: " << args[1] << endl;

  XmlModify modify = env.db().createModify();
  env.results()->reset();

  unsigned int modifications;
  if(env.txn()) {
	  XmlTransaction myTxn = env.childTransaction();
	  modify.addRemoveStep(env.db().prepare(myTxn, args[1], env.context()));
	  modifications = modify.execute(myTxn, *env.results(), env.context(), env.uc());
	  myTxn.commit();
  }
  else {
	  modify.addRemoveStep(env.db().prepare(args[1], env.context()));
	  modifications = modify.execute(*env.results(), env.context(), env.uc());
  }

  if(env.verbose()) cout << modifications << " modifications made." << endl;
}

